;  Originally based on MDS-800 I/O Drivers
;  for CP/M 2.2
 (4 drive single density version)
;
;  Version 2.2 February, 1980
;
;  Copyright (c) 1980
;  Digital Research
;  Box 579, Pacific Grove
;  California, 93950
;
;
  Modified for VersaFloppy II
;  Controller, using 3.5" pc drive
;
;
;  "BIAS" is address offset from 3400H
;   memory systems bigger than 20K
;
MSIZE:	 equ	 20		; cp/m mem size (Kbytes)
;
BIAS:	 equ	 (MSIZE-20)*1024
;
CPMB:    equ     3400h+BIAS 	; base of CCP
BDOS:    equ     806h+CPMB      ; basic resident dos
BIOS:	 equ	 CPMB+1600h
CPML:    equ     BIOS-CPMB  	; length of cpm system
NSECTS:  equ     (CPML/128)     ; # of sectors to load
offset:  equ     2       	; # of cp/m system tracks
cdisk:   equ     0004h   	; addr of last logged disk 
iobyte:  equ	 0003h		; intel i/o byte
buff:    equ     0080h   	; default buffer address
;				; for sector read/write
retry:   equ     5      	; max retries on disk i/o
ndisks:  equ     4
;
;
DCMD:     equ      63H    ; DMA disk control port
W:        equ      64H    ; WDC 179X Base ADDRESS
WCMD:     equ      W+0    ; COMMAND PORT
WSTAT:    equ      W+0    ; STATUS PORT
WTRACK:   equ      W+1    ; TRACK REG
WSECT:    equ      W+2    ; SECTOR REG
WDATA:    equ      W+3    ; DATA I/O REG
WRESET:   equ      60h    ; Reset controller with 0ffH
;
;
WHOME:    equ      08H 	   ; HOME COMMAND
WREAD:    equ      88H	   ; READ SECTOR COMMAND
WWRITE:   equ      0a8H	   ; WRITE SECTOR COMMAND
WSEEK:    equ      18H     ; SEEK TO GIVEN TRACK
WUNLD:    equ      10H	   ; SEEK AND UNLOAD HEAD
WLOAD:    equ      18H	   ; SEEK AND LOAD HEAD
WTRK0:    equ      0bH     ; Set Track 0
WSTEPIN:  equ	   58H	   ; go 1 Trk towards 76
WSTEPOUT: equ	   78H	   ; go 1 Trk towards 0
;       
;
MONITOR:  equ     0e100h  
POLL:     equ     0e103h    ; console in to reg-a
CPMOUT:   equ     0e138h    ; console char from c out
;
;tms:       equ   0e0H
;tmsserin:  equ	  tms		; serial in
;tmsparin:  equ   tms+1		; parallel in
;tmsirqadr: equ   tms+2		; irq address
;tmsstatus: equ   tms+3		; status
;tmsserout: equ   tms+6		; serial out
;tmsmask:   equ   tms+8		; mask
;
cr:      equ     0dh     ; carriage return
lf:      equ     0ah     ; line feed
;
dpblk0:	 equ	dpblk
dpblk1:	 equ	dpblk
dpblk2:	 equ	dpblk
dpblk3:	 equ	dpblk
;
;
; ************************************************
;
;   perform following functions
;   boot    cold start
;   wboot   warm start (save i/o byte)
;   (boot and wboot are the same for mds)
;   const   console status
;           reg-a = 00 if no character ready
;           reg-a = ff if character ready
;   conin   console character in (result in reg-a)
;   conout  console character out (char in reg-c)
;   list    list out (char in reg-c)
;   punch   punch out (char in reg-c)
;   reader  paper tape reader in (result to reg-a)
;   home    move to track 00
;
;   (the following calls set-up the io parameter block
;   for the
 mds, which is used to perform subsequent 
;   reads and writes)
.
;   seldsk  select disk given by reg-c (0,1,2...)
;   settrk  set track address (0,...76) for read/write
;   setsec  set sector address (1,...,26) for read/write
;   setdma  set subsequent dma address (initially 80h)
;
;   (read/write assume previous calls set up the parameters)
;   read    read track/sector to preset dma address
;   write   write track/sector from preset dma address
;
;
	org     BIOS
;
;   jump vector for indiviual routines
;
	jmp     boot
wboote: jmp     wboot
	jmp     const
	jmp     conin
	jmp     conout
	jmp     list
	jmp     punch
	jmp     reader
	jmp     home
	jmp     seldsk
	jmp     settrk
	jmp     setsec
	jmp     setdma
	jmp     read
	jmp     write
	jmp     listst  	; list status
	jmp     sectran
;
;   end of controller - independent code, the remaining
;   subroutines
 are tailored to the particular operating 
;   environment.
;
;   The following code assumes a monitor exists at 0e100h
;   and uses the i/o subroutines within the monitor
;
;   fixed data tables for four-drive standard
;   IBM-compatible 8" disks
;   disk parameter header for disk 00
;
dpbase:	dw	trans
	dw	0000H
	dw	0000H
	dw	0000H
	dw	dirbf
	dw	dpblk0
	dw	chk00
	dw	all00
;	disk parameter header for disk 01
	dw	trans
	dw	0000H
	dw	0000H
	dw	0000H
	dw	dirbf
	dw	dpblk1
	dw	chk01
	dw	all01
;	disk parameter header for disk 02
	dw	trans
	dw	0000H
	dw	0000H
	dw	0000H
	dw	dirbf
	dw	dpblk
2
	dw	chk02
	dw	all02
;	disk parameter header for disk 03
	dw	trans
	dw	0000H
	dw	0000H
	dw	0000H
	dw	dirbf
	dw	dpblk3
	dw	chk03
	dw	all03
;
;  sector translate vector
. NOT used by system
;  read/write calls.
;
trans:	db	1,7,13,19	;sectors 1,2,3,4
	db	25,5,11,17	;sectors 5,6,7,8
	db	23,3,9,15	;sectors 9,10,11,12
	db	21,2,8,14	;sectors 13,14,15,16
	db	20,26,6,12	;sectors 17,18,19,20
	db	18,24,4,10	;sectors 21,22,23,24
	db	16,22		;sectors 25,26
;
;
;
  disk parameter block, common to all disks
;
dpblk:	dw	26		;sectors per track
	db	3		;block shift factor
	db	7		;block mask
	db	0		;null mask
	dw	242		;disk size-1
	dw	63		;directory max
	db	192		;alloc 0
	db	0		;alloc 1
	dw	16		;check size
	dw	2		;track offset
;
;
;    end of fixed tables
;
 **************************************
;
; signon message: xxk cp/m vers y.y
;
signon: db      cr,lf,lf
	db      '20'    	; 20k bios
	db      'K CP/M vers '
	db      '2.2'
	db      cr,lf,0
;
; print signon message and go to ccp
; OK  8/05
;
boot:   lxi     sp, buff+80H
	lxi     h, signon
	call    prmsg   	; print message
	xra     a       	; clear accumulator
	sta	iobyte		; clear the iobyte
	sta     cdisk   	; set initially to disk a
	mvi	a, retry	; max retries
	sta 	errcount
;	call    home    	; reset to trk 0
	jmp     gocpm   	; go to cp/m
;
;
 simplest case: all sectors loaded
 from disk
;
wboot:	lxi	sp, buff+80H   
	mvi	a, retry	; max retries
	sta 	errcount
;	lxi	h, 6000h	; for REPORT debug
;	shld	RPTCNT
;
COLD:   xra    a
	sta    iot	  ; initial trk	
	lxi    b, 2       ; b=0 c=2 (sector)
	mvi    a, 7eH     ; Drive 0, side 0, 8" 
	out    DCMD 
	mvi    d, NSECTS
	lxi    h, CPMB
	  ; 3400H + BIAS
	mvi    a, WTRK0   ; set Track 0
	call   WRTCMD

;
; get CCP and BDOS
;
LSECT:  call   setsec	  ; set sector
	call   shread	  ; read sector
	cpi    0	  ; check for err
	jnz    booterr    ; z=0 -> a=1-> error 
	dcr    d          ; next sector
	jnz    NXSECT
	mvi    a, 0ffH
	out    WRESET 	  ; reset controller
	jz     boot       ; sign on, goto CP/M
;
; More sectors to load        
;
NXSECT:	inr    c          ; next sector
	mov    a, c
	cpi    27         ; last sector of track?
	jc     LSECT      ; no, read another
;
; end of track 0, goto trk 1
;
	mvi    c, 1
	call   settrk
	jmp    LSECT

;
; **********************************
;
; error occurred, print message and retry
;
booterr:
 lda	errcount    ; recall counts
	 dcr    a
	 jz     booter0
	 sta    errcount    ; try again
	 jmp    COLD
;
;  otherwise too many retries
;
booter0:
 lxi	h, bootmsg
	 call	prmsg
	 call	POLL
	 jmp	MONITOR     ; S-100 monitor
;
bootmsg:
 db     '?boot', 0
;
; ******************************************
;
;  END load operation, set parameters, go to 
;  CP/M. (enter here from cold start boot)
;
; ******************************************
;
;  reset monitor entry points
;
gocpm:  di
	mvi     a, 0c3H
	sta     0
	lxi     h, wboote
	shld    1       	; jmp wboot at loc 00
	sta     5
	lxi     h, BDOS
	shld    6       	; jmp BDOS at loc 5
	sta     7*8     	; jmp to MONITOR
	lxi     h, MONITOR
	shld    7*8+1
;
;  set default buffer address to 80h
;
	lxi     b, buff
	call    setdma
;
	mvi     c, 2
	call    settrk
	lda     cdisk   	; last logged disk number
	mov     c, a     	; send to ccp to log it in
;	ei
	jmp     CPMB
;
; ************************************
; console status to reg-a
; 00 = no irq pending: ff = irq pending
;
const:  mvi	a, 0h
	ret
;
; console character to reg-a
;
conin:  jmp    POLL
;	ret
;
; console char from c to console out
; CPMOUT in Monitor moves c -> a
;
conout: call    CPMOUT
	ret
;
; *******************************
;
; list device out
;
list:	mov	a, c	  ; char to reg a
	ret		  ; null subroutine
;
; return list status
;
listst:
 xra     a
	ret               ; always not ready
;
; punch device out
;
punch:  mov	a, c	  ; char to reg a
	ret	 	  ; null subroutine
;
; reader character in to reg-a
;
reader: mvi	a, 1ah	  ; end of file
	ani	7fh	  ; strip parity bit
	ret
;
;
; ****************************
;  Disk routines
; ****************************
;
; move to home position
; treat as track 00 seek
;
home:   xra     a
	sta     iot
		; track #
	mvi     a, WTRK0	; set trk 0
	call	WRTCMD		; write to controller
	ret
	
;
; select disk given by register c
;
seldsk: lxi     h, 0000h 	; return 0000 if error
	mov     a, c 
	cpi     ndisks  	; too large?
	rnc             	; leave HL = 0000
;
;  for now there is only 1 drive
;
	mvi     a, 0ffH
	out     WRESET 		; reset controller
;
	mvi     a, 7eH		; Drive 0, side 0, 8"
	out	DCMD
	xra     a
;	
setdrive:
 mov     b, a     	; save the function
	mov     l, c 
	mvi     h, 0     	; HL = disk number
	dad     h        	; * 2
	dad     h       	; * 4
	dad     h       	; * 8
	dad     h       	; * 16
	lxi     d, dpbase
	; disk header table 
	dad     d       	; HL = dpbase+(diskno*16)
	ret
;
;
; set to track # given by c
	
;
settrk: push    b
	push    d
	lda     iot
	mov     b, a		; current trk
	mov	d, c		; new trk
	call    DELAY
	in      WTRACK		; trk from controller
	in      WTRACK		; trk from controller
	cmp     b
	cnz	FIXIT
	cmp     c 		; iot - new
	jz	stexit		; iot = new
	jc	mvin		; iot < new
;				  iot > new
	sub     c		; iot - new -> a
	mov	b, a
;	
mvout1: mvi     a, WSTEPOUT	; step toward 0 
	call	WRTCMD		; to desired trk
	dcr	b
	jnz	mvout1 
	jmp	stexit
;	
mvin:   mov     b, a		; iot-> b
	mov     a, c		; new-> a 
	sub     b		; new - iot>0 in a
	mov     b, a
mvin1:	call	MOVIN
	dcr	b
	jnz	mvin1
;
stexit: mov	a, d
	sta	iot		; update
	pop	d
	pop	b
	ret
;
; b has iot contents
;
FIXIT: 	mvi     a, WTRK0	; set trk 0
	call	WRTCMD	
FX1:	call    MOVIN
	dcr     b
	jnz     FX1
	ret
;
MOVIN:	mvi     a, WSTEPIN	; step toward 76
	call	WRTCMD
	ret
;
;
; set sector number given by c
;
setsec: mov	a, c
        sta	ios
	; sector #
	out	WSECT
	ret
;
; translate sector bc using table at de
; NOT used for read/write system tracks
;
sectran:
 mvi     b, 0    ; double precision sector # in BC
	 xchg            ; translate table address to HL
	 dad     b       ; translate(sector) address
	 mov     a, m    ; translated sector number to A
	 sta     ios
	 mov     l, a    ; return sector number in L
	 ret
;
; set dma address given by regs b,c
; this is buffer for 1 sector read/write
;
setdma: mov     l, c
	mov     h, b
	shld    iod
	ret
;
; ******************************
;  disk read function
; ******************************
; (assuming disk/trk/sec/dma set)
;  no erros, a=0;  error, a=1
;
read:   lhld	iod	   ; get buffer address
shread:	mvi     a, WREAD   ; read
	call	WRTCMD	
	mvi	e, 080H    ; bytes/sector   
read1:  in	WDATA      ; need 2 for 
	in	WDATA      ; timing
	mov	m, a
	inx	h
	dcr	e
	jnz	read1
	call	CHKSTAT	    ; a=0, no error
;	ora	a
;	rz
;	mvi     a, 1
	ret
;
; ******************************
;  disk write function
; ******************************
;
write:  lhld	iod	    ; get buffer address
	mvi     a, WWRITE   ; write function
	
call	WRTCMD
	mvi	e, 080H     ; bytes/sector   
write1:	mov	a, m
	out	WDATA       ; need 2 for 
	out	WDATA       ; timing
	inx     h
	dcr     e
	jnz     write1
	call	CHKSTAT	    ; a=0, no error
	ora	a
	rz
	mvi     a, 1
	ret
;
;
; **************************
; utility subroutines
; **************************
;
; print message at h,l to 0
;
prmsg:  mov     a, m
	ora     a       	; zero?
	rz
; more to print
	push    h
	mov     c, a
	call    conout
	pop     h
	inx     h
	jmp     prmsg
;
; ******************************
;
DELAY:  mvi	a, 0c8H  ; wait for valid status
WAIT:   dcr	a
	jnz	WAIT
DL1:    in	WSTAT    ; GET STATUS
	ani	1        ; BUSY (y/n)
	jnz	DL1      ; YES,KEEP WAITING
	ret
; 
; ******************************
;  Check controller status for errors
;  ret a=0 if none, else a=1
;
CHKSTAT: 
;	call	REPORT	    ; for debug
	call    DELAY
	in      WSTAT       ; Error check
	ani     9dH         ; Read success -> 0
	jnz     CHST1	    ; read error 
	xra     a	    ; no errors
	ret
CHST1:  mvi     a, 1
	ret
;
; ******************************
; Write command to controller
; ******************************
;
WRTCMD:	push    psw
	
call	DELAY
	pop	psw
	out	WCMD
	ret
;
; ******************************
; Print track and sector 
; ******************************
;
;PNTDATA:  equ   0e775H
;
;REPORT: push    b
;	push    d
;	push    h
;	lxi     h, TRKMSG
;	call    prmsg
;	lda     iot
;	call    PNTDATA
;	lxi     h, SCTMSG
;	call    prmsg
;	lda     ios
;	call    PNTDATA
;	mvi     c, 0dh
;	call    conout
;	mvi     c, 0ah
;	call    conout
;	pop     h
;	pop     d
;	pop     b
;	ret
;
;TRKMSG: db     ' T=', 0
;SCTMSG: db     '  S=', 0
;
;
; *******************************
;  data areas (must be in ram)
; *******************************
;
errcount: db    0	; count for retries
;RPTCNT:	  dw	6000h
;
;  io parameter block
;
iof:    db      WREAD   ; io function, initial read
ion:    db      1       ; number of sectors to read
iot:    db      offset  ; track number
ios:    db      1       ; sector number
iod:    dw      buff    ; io address
;
;
;   scratch ram area for BDOS use
;
begdat:	equ	$	; beginning of data area
dirbf:	ds	128	; scratch directory area
all00:	ds	31	; allocation vector 0
all01:	ds	31	; allocation vector 1
all02:	ds	31	; allocation vector 2
all03:	ds	31	; allocation vector 3
chk00:	ds	16	; check vector 0
chk01:	ds	16	; check vector 1
chk02:	ds	16	; check vector 2
chk03:	ds	16	; check vector 3
;
enddat:  equ	$		; end of data area
datsiz:  equ	$-begdat	; size of data area
