;  PUTSYS - Code to write cpm in memory to floppy tracks 0, 1
;
;   Based on MDS-800 Cold Start Loader for CP/M 2.0
;   Version 2.0 August, 1979
;
;
MSIZE:   equ     20              ; min mem size in kbytes
BIAS:    equ     (MSIZE-20)*1024 ; offset from 20K
CCP:     equ     3400H+BIAS      ; base of CPP
BDOS:    equ     CCP+806H        ; entry to dos for calls
BIOS:    equ     CCP+1600H 
BIOSL:   equ     0300H           ; length of bios
MONITOR: equ     0e100H
SIZE:    equ     BIOS+BIOSL-CCP  ; size of cp/m
SECTS:   equ     SIZE/128        ; # of sectors to load (~56)
;
;
;
; *********************************************
;             FLOPPY DISK EQUATES.
; *********************************************
;
DCMD:     equ      63H    ; DMA disk control port
W:        equ      64H    ; WDC 179X Base ADDRESS
WCMD:     equ      W+0    ; COMMAND PORT
WSTAT:    equ      W+0    ; STATUS PORT
WTRACK:   equ      W+1    ; TRACK REG
WSECT:    equ      W+2    ; SECTOR REG
WDATA:    equ      W+3    ; DATA I/O REG
WRESET:   equ      60H	  ; Reset controller with 0ffH
;
;
;************************************************
;       COMMON 179X CONTROLLER COMMANDS
;************************************************
;
WHOME:    equ      08H 	     ; HOME COMMAND
WREAD:    equ      88H	     ; READ SECTOR COMMAND
WWRITE:   equ      0a8H	     ; WRITE SECTOR COMMAND
WSEEK:    equ      18H       ; SEEK TO GIVEN TRACK COMMAND
WUNLD:    equ      10H	     ; SEEK AND UNLOAD HEAD COMMAND
WLOAD:    equ      18H	     ; SEEK AND LOAD HEAD COMMAND
WTRK0:    equ      0bH       ; Set Track 0
WSTEPIN:  equ	   58H	     ; go 1 Trk towards 76
;       
; ************************************************
;
	org     0b000H
;
	jmp   COLD
;
; ******************************
;
DELAY:  mvi   e, 0c8H  ; wait for valid status
WAIT:   dcr   e
	jnz   WAIT
NB:     in    WSTAT    ; GET STATUS
	ani   1        ; BUSY (y/n)
	jnz   NB       ; YES,KEEP WAITING
	ret
;	
; ******************************
;	
COLD:   lxi   b, 2       ; b=0 c=2 (sector)
	mvi   a, 7eH     ; Drive 0, side 0, 8" 
	out   DCMD 
	mvi   d, SECTS
	lxi   h, CCP
	call  DELAY
	mvi   a, WTRK0   ; set Track 0
	out   WCMD

;
;
; load next sector
; Track in B, Sector in C, Total sects in D
; Mem address in (H,L), use E as scratch
; branch to COLD if error
;
LSECT:  mov   a, c
	out   WSECT      ; Set sector
	call  DELAY
SECTOR: mvi   a, WWRITE  ; write sector
	out   WCMD
	mvi   e, 080H    ; bytes/sector   
SECT1: 	mov   a, m
	out   WDATA      ; need 2 for 
	out   WDATA      ; timing
	inx   h
	dcr   e
	jnz   SECT1
	call  DELAY
	in    WSTAT      ; Error check
	ani   9dH        ; Read success -> 0
	jnz   COLD       ; read error 
;        
	dcr   d          ; next sector
	jnz   NXSECT
	mvi   a, 0ffH
	out   WRESET	 ; reset controller
	jmp   MONITOR
;
; More sectors to load        
;
NXSECT:	inr   c          ; next sector
	mov   a, c
	cpi   27         ; last sector of track?
	jc    LSECT      ; no, read another
;
; end of track, goto next one
;
	mvi   c, 1       
	call  DELAY      ; wait for disk 
	mvi   a, WSTEPIN ; go 1 track -> 76
	out   WCMD
	jmp   LSECT
